import { Card, useMutation, useStorage } from '@/app/liveblocks.config';
import { shallow } from '@liveblocks/client';
import { ReactSortable } from 'react-sortablejs';
import NewCardForm from './forms/NewCardForm';


type ColumnProps = {
    id: string;
    name: string;
}


const Column = ({id, name}: ColumnProps) => {
    const columnCards = useStorage<Card[]>(root => {
        return root.cards.filter(card => card.columnId === id).map(c => ({...c})).sort((a, b) => a.index - b.index);
    }, shallow)

    const updateCard = useMutation(({storage}, index, updateData) => {
        const card = storage.get('cards').get(index);
        if(card){
            for(let key in updateData){
                card?.set(key as keyof Card, updateData[key]);
            }
        }
    }, []);

    const setTasksOrderForColumn = useMutation(({storage}, sortedCards: Card[], newColumnId) => {
        const idsOfSortedCards = sortedCards.map(c => c.id.toString());
        const allCards: Card[] = [...storage.get('cards').map(c => c.toObject())]
        idsOfSortedCards.forEach((sortedCardId, colIndex) => {
            const cardStorageIndex = allCards.findIndex(c => c.id.toString() === sortedCardId);
            updateCard(cardStorageIndex, {
                columnId: newColumnId,
                index: colIndex,
            })
        })

    },[])


  return (
    <div className='w-50 shadow-md rounded-md p-2 bg-gray-100'>
        <h3>{name}</h3>
        {columnCards && (
            <>
            <ReactSortable list={columnCards} setList={items => setTasksOrderForColumn(items, id)} group="cards" ghostClass='opacity-40' className='min-h-12'>
                {columnCards.map(card => (
                    <div key={card.id} className='border my-2 p-2 rounded-sm bg-white'>
                        <span>{card.name}</span>
                    </div>
                ))}
            </ReactSortable>  
           </> 
        )}
        <NewCardForm columnId={id}/>
    </div>

  )
}

export default Column